<?php
/**
 * Video Tree Class.
 *
 * $Horde: incubator/video/lib/Categories.php,v 1.2 2007/06/28 09:02:31 duck Exp $
 *
 * Copyright 2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Duck <duck@obala.net>
 * @package Video
 */
class Video_Categories {

    /**
     * Hash containing connection parameters.
     *
     * @var array
     */
    private $_params = array();

    /**
     * Handle for the current database connection.
     *
     * @var DB
     */
    private $_db;

    /**
     * Handle for the current database connection, used for writing. Defaults
     * to the same handle as $db if a separate write database is not required.
     *
     * @var DB
     */
    private $_writedb;

    /**
     * An array containing all the tree nodes.
     *
     * @var array
     */
    private $_nodes = array();

    /**
     * The top-level nodes in the tree.
     *
     * @var array
     */
    private $_root_nodes = array();

    /**
     * An enumeratic array
     *
     * @var array
     */
    private $_enum = array();

    /**
     * Has for view url link
     *
     * @var array
     */
    private $_view_url;

    /**
     * Handle for the tables prefix.
     *
     * @var prefix
     */
    private $prefix = 'video';

    /**
     * Contruct the Video object
     */
    public function __construct($categoreis = null)
    {
        $categoreis = $this->getCategories(false);
        foreach ($categoreis as $id => $category) {
            $this->_nodes[$id]['label'] = $category['category_name'];
            if (empty($category['category_parentid'])) {
                if (!in_array($id, $this->_root_nodes)) {
                    $this->_root_nodes[] = $id;
                }
            } else {
                if (empty($this->_nodes[$category['category_parentid']]['children'])) {
                    $this->_nodes[$category['category_parentid']]['children'] = array();
                }
                if (!in_array($id, $this->_nodes[$category['category_parentid']]['children'])) {
                    $this->_nodes[$category['category_parentid']]['children'][] = $id;
                }
            }
        }

        $this->_buildIndents($this->_root_nodes);
    }

    /**
     * Returns the string for category selection
     *
     * @return string
     */
    public function getSelect()
    {
        $output = '';
        foreach ($this->_root_nodes as $node_id) {
            $output .= $this->_buildSelect($node_id);
        }

        return $output;
    }

    private function _buildSelect($node_id)
    {
        $output = '<option value="' . $node_id . '">'
                . str_repeat(' - ', $this->_nodes[$node_id]['indent'])
                . $this->_nodes[$node_id]['label'] . '</option>';

        if (isset($this->_nodes[$node_id]['children'])) {
            $num_subnodes = count($this->_nodes[$node_id]['children']);
            for ($c = 0; $c < $num_subnodes; $c++) {
                $child_node_id = $this->_nodes[$node_id]['children'][$c];
                $output .= $this->_buildSelect($child_node_id);
            }
        }

        return $output;
    }

    /**
     * Returns the enumeratic array for select form pameter.
     *
     * @return array  The category array (cat => name)
     */
    public function getEnum()
    {
        if (empty($this->_enum)) {
            foreach ($this->_root_nodes as $node_id) {
                $this->_buildEnum($node_id);
            }
        }

        return $this->_enum;
    }

    private function _buildEnum($node_id)
    {
        $this->_enum[$node_id] = str_repeat(' - ', $this->_nodes[$node_id]['indent'])
                                . $this->_nodes[$node_id]['label'];

        if (isset($this->_nodes[$node_id]['children'])) {
            $num_subnodes = count($this->_nodes[$node_id]['children']);
            for ($c = 0; $c < $num_subnodes; $c++) {
                $child_node_id = $this->_nodes[$node_id]['children'][$c];
                $this->_buildEnum($child_node_id);
            }
        }
    }

    /**
     * Returns html for category selection
     *
     * @return string
     */
    public function getHtml()
    {
        $this->_view_url = Horde::applicationUrl('list.php');

        $output = '';
        foreach ($this->_root_nodes as $node_id) {
            $output .= $this->_buildHtml($node_id);
        }

        return $output;
    }

    private function _buildHtml($node_id)
    {
        $output = '';
        if ($this->_nodes[$node_id]['indent'] == 0) {
            $output .= '<strong>';
        }
        $url = Util::addParameter($this->_view_url, 'ad_category', $node_id);
        $output .= Horde::link($url) . $this->_nodes[$node_id]['label'] . '</a>, ';
        if ($this->_nodes[$node_id]['indent'] == 0) {
            $output .= '</strong><br />';
        }

        if (isset($this->_nodes[$node_id]['children'])) {
            $num_subnodes = count($this->_nodes[$node_id]['children']);
            for ($c = 0; $c < $num_subnodes; $c++) {
                $child_node_id = $this->_nodes[$node_id]['children'][$c];
                $output .= $this->_buildHtml($child_node_id);
            }
        }

        return $output;
    }

    /**
     * Set the indent level for each node in the tree.
     */
    private function _buildIndents($nodes, $indent = 0)
    {
        foreach ($nodes as $id) {
            $this->_nodes[$id]['indent'] = $indent;
            if (!empty($this->_nodes[$id]['children'])) {
                $this->_buildIndents($this->_nodes[$id]['children'], $indent + 1);
            }
        }
    }

    /**
     * Returns array of cateogriy children id
     *
     * @return array
     */
    public function getChildren($id)
    {
        if (!empty($this->_nodes[$id]['children'])) {
            return $this->_nodes[$id]['children'];
        } else {
            return array();
        }
    }

    /**
     * Returns the current language
     *
     * @return string  The current language.
     */
    public function getAllowed($perm = PERMS_SHOW)
    {
        $cats = $this->getCategories();
        if (Auth::isAdmin('Video:admin') || Auth::isAdmin('Video:admin') ||
            $GLOBALS['perms']->hasPermission('Video', Auth::getAuth(), $perm)) {
            return $cats;
        }

        foreach ($cats as $key => $value) {
            if (!$GLOBALS['perms']->hasPermission('Video:categories', Auth::getAuth(), $perm)  &&
                !$GLOBALS['perms']->hasPermission('Video:categories:' . $value, Auth::getAuth(), $perm)) {
                unset($cats[$key]);
            }
        }

        return $cats;
    }

    /**
     * Returns the string for category selection
     *
     * @return string
     */
    public function getName($id)
    {
        $cats = $this->getCategories();
        return $cats[$id];
    }

    /**
     * Save a category data into the backend from edit form.
     *
     * @param array $info  The category data to save.
     *
     * @return mixed  PEAR error.
     */
    public function saveCategory($info)
    {
        $this->_connect();

        /* Update/Insert category. */
        if (!empty($info['category_id'])) {
            $result = $this->_updateCategory($info['category_id'], $info);
            if ($result instanceof PEAR_Error) {
                return $result;
            }
        } else {
            $info['category_id'] = $this->_insertCategory($info);
            if ($info['category_id'] instanceof PEAR_Error) {
                return $info['category_id'];
            }
        }

        /* If image uploaded save to backend. */
        if (!empty($info['category_image']['name'])) {
            $image = $GLOBALS['video']->saveImage($info['category_image']['file'], $info['category_id'], 'categories', $info['image_resize']);
            if ($image instanceof PEAR_Error) {
                return $image;
            }

            $sql = 'UPDATE ' . $this->prefix . '_categories SET category_image = ? WHERE category_id = ?';
            $this->_writedb->query($sql, array(1, $info['category_id']));
        }

        // Clean cache
        $GLOBALS['cache']->expire('VideoCategories0');
        $GLOBALS['cache']->expire('VideoCategories1');

        return $info['category_id'];
    }

    /**
     * Insert category data.
     *
     * @param mixed $data  The category data to insert.
     *
     * @return array  Inserted ID or PEAR error.
     */
    private function _insertCategory($data)
    {
        $new_id = $this->_writedb->nextId('Video_categories');

        $sql = 'INSERT INTO ' . $this->prefix . '_categories' .
               ' (category_id, category_name, category_parentid, category_description)' .
               ' VALUES (?, ?, ?, ?)';
        $values = array($new_id,
                        $data['category_name'],
                        (int)$data['category_parentid'],
                        $data['category_description']);

        $category = $this->_writedb->query($sql, $values);
        if ($category instanceof PEAR_Error) {
            Horde::logMessage($category, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $category;
        }

        return $new_id;
    }

    /**
     * Update category data.
     *
     * @param integer $category_id  The category id to update.
     * @param array   $data       The category data to update.
     *
     * @return array  NULL or PEAR error.
     */
    private function _updateCategory($category_id, $data)
    {
        $sql = 'UPDATE ' . $this->prefix . '_categories' .
               ' SET category_name = ?, category_parentid = ?, ' .
               ' category_description = ? WHERE category_id = ?';
        $values = array($data['category_name'],
                        (int)$data['category_parentid'],
                        $data['category_description'],
                        $category_id);

        $category = $this->_writedb->query($sql, $values);
        if ($category instanceof PEAR_Error) {
            Horde::logMessage($category, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $category;
        }
    }

    /**
     * Delete a category
     *
     * @return booelan
     */
    public function deleteCategory($id)
    {
        // Clean cache
        $GLOBALS['cache']->expire('VideoCategories0');
        $GLOBALS['cache']->expire('VideoCategories1');

        $this->_connect();
        $GLOBALS['video']->deleteImage($id, 'categories');
        $sql = 'DELETE FROM ' . $this->prefix . '_categories WHERE category_id = ?';
        return $this->_db->query($sql, array($id));
    }

    /**
     * Get available categories
     *
     * @return array  An array containing caegories
     */
    public function getCategories($flat = true)
    {
        $cache_key = 'VideoCategories' . $flat;
        $categories = $GLOBALS['cache']->get($cache_key, $GLOBALS['conf']['cache']['default_lifetime']);
        if ($categories) {
            return unserialize($categories);
        }

        $this->_connect();

        $sql = 'SELECT category_id, category_name, category_parentid, category_description '
            . ' FROM ' . $this->prefix . '_categories ORDER BY category_name ASC';
        $result = $this->_db->getAssoc($sql, false, array(), DB_FETCHMODE_ASSOC);

        if ($result instanceof PEAR_Error) {
            return $result;
        }

        if (!$flat) {
            $GLOBALS['cache']->set($cache_key, serialize($result));
            return $result;
        }

        $categories = array();
        foreach ($result as $category_id => $row) {
            if (!empty($row['category_description']) && $row['category_name'] != $row['category_description']) {
                $row['category_name'] .=  ' [ ' . $row['category_description'] . ' ]';
            }
            $categories[$category_id] = $row['category_name'];
        }

        $GLOBALS['cache']->set($cache_key, serialize($categories));
        return $categories;
    }

    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return boolean  True on success; exits (Horde::fatal()) on error.
     */
    private function _connect()
    {
        $this->_db = $GLOBALS['video']->db;
        $this->_writedb = $GLOBALS['video']->writedb;
    }
}
