<?php

require 'Horde/View/Base.php';
require 'Horde/View.php';

/**
 * Video General View Class
 *
 * Copyright 2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Duck <duck@obala.net>
 * @package Video
 */
class Video_View extends Horde_View {

    /**
     * Constructor
     */
    public function __construct()
    {
        /* Set parents defualt data */
        parent::__construct(array('templatePath' => VIDEO_TEMPLATES,
                                  'encoding' => NLS::select()));
    }

    /**
     * Formats time according to user preferences.
     *
     * @param int $timestamp  Message timestamp.
     *
     * @return string  Formatted date.
     */
    public function format_date($timestamp)
    {
        return strftime($GLOBALS['prefs']->getValue('date_format'), $timestamp);
    }

    /**
    * Format file size
    *
    * @param int filesize
    *
    * @return boolean formatted filesize.
    */
    public function format_filesize($filesize)
    {
        $units = array('B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB');
        $pass = 0; // set zero, for Bytes
        while ($filesize >= 1024) {
            $filesize /= 1024;
            $pass++;
        }

        return round($filesize, 2) . ' ' . $units[$pass];
    }

    /**
     * Formats time according to user preferences.
     *
     * @param int $timestamp  Message timestamp.
     *
     * @return string  Formatted date.
     */
    public function format_datetime($timestamp)
    {
        return strftime($GLOBALS['prefs']->getValue('date_format'), $timestamp)
            . ' '
            . (date($GLOBALS['prefs']->getValue('twentyFour') ? 'G:i' : 'g:ia', $timestamp));
    }

    /**
    * Format movie lenght
    *
    * @param int seconds
    *
    * @return boolean formatted time.
    */
    public function format_len($len)
    {
        $hours = round($len/3600);
        $minutes = round(($len % 3600 - $len % 60)/60);
        $seconds = round($len % 60);

        if ($hours < 10) {
            $hours = '0' . $hours;
        }

        if ($minutes < 10) {
            $minutes = '0' . $minutes;
        }

        if ($seconds < 10) {
            $seconds = '0' . $seconds;
        }

        return "$hours : $minutes : $seconds";
    }

    /**
    * Get video thumbnail
    *
    * @param int $id Video id
    * @param int $status Video status
    */
    public function getThumbnail($id, $status)
    {
        if ($status != Video::PROCESSING) {
            return $GLOBALS['video']->getWebDIR($id) . '/00000001.jpg';
        } else {
            return $GLOBALS['registry']->getImageDir() . '/video.png"';
        }
    }

    /**
    * Get video view link
    *
    * @param int $id Video id
    */
    public function getViewLink($id)
    {
        static $view_url;

        if ($view_url === null) {
            $view_url = Horde::applicationUrl('video.php');
        }

        $dir = $GLOBALS['video']->getWebDIR($id);
        $html = '<a href="' . Util::addParameter($view_url, 'id', $id) . '" title="';
        for ($i = 1; $i<=$GLOBALS['conf']['converting']['params']['thumb_num']; $i++) {
            $html .= '<img src=\'' . $dir . '/0000000' . $i . '.jpg\' />';
            if ($i%2 == 0) {
                $html .= '<br />';
            }
        }
        $html .= '"><img src="' . $dir . '/00000001.jpg"></a>';

        return $html;
    }

}
