<?php
/**
 * $Horde$
 *
 * Copyright 2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author Jernej Strasner <masterboi@gmail.com>
 * @author Duck <duck@obala.net>
 */
@define('VIDEO_BASE', dirname(__FILE__) . '/..');
require_once VIDEO_BASE . '/lib/base.php';
require_once VIDEO_BASE . '/lib/Forms/Video.php';

/* Only adss should be using this. */
if (!Auth::isAuthenticated()) {
    Horde::authenticationFailureRedirect();
}

$vars = Variables::getDefaultVariables();
$id = $vars->get('id');
if ($id) {
    $movie = $video->getVideo($id);
    if ($movie['user'] != Auth::getAuth() && !Auth::isAdmin('video:admin')) {
        $notification->push(sprintf(_("You are not alowed to edit video %s."), $id), 'horde.warning');
        header('Location: ' . Horde::applicationUrl('videos/index.php'));
        exit;
    }

    if ($vars->get('formname') === null) {
        $vars->set('title', $movie['title']);
        $vars->set('description', $movie['description']);
        $vars->set('category', $movie['category']);
        $vars->set('tags', $movie['tags']);
        $vars->set('embed', $movie['embed']);
    }
}

$title = $id ? _("Edit Video") : _("Add Video");
$form = new Video_Form($vars, $title);

if ($form->validate()) {

    $form->getInfo(null, $info);
    $id = $video->save($info);

    if ($id instanceof PEAR_Error) {
        $notification->push($result->getMessage(), 'horde.error');
    } else {
        $message = $id ? _("The video \"%s\" data has been updated sucessfully.") :
            _("The video \"%s\" data has been added successfully.");
        $notification->push(sprintf($message, $info['title']), 'horde.success');

        $uploaded = Browser::wasFileUploaded('video_file');
        if ($uploaded instanceof PEAR_Error) {
            if ($uploaded->getCode() != UPLOAD_ERR_NO_FILE) {
                $notification->push(_("But there was an error with the uploaded video: ") . $uploaded->getMessage(), 'horde.warning');
            } else {
                header('Location: ' . Horde::applicationUrl('videos/index.php'));
                exit;
            }
        } elseif ($uploaded) {
            $result = $video->addFile($id, $info['video_file']);
            if ($result instanceof PEAR_Error) {
                $notification->push(_("But there was an error with the uploaded video: ") . $result->getMessage(), 'horde.warning');
            } else {
                $notification->push(_("Your video was successfuly uploaded. Processing video can take a few minutes."), 'horde.success');
                header('Location: ' . Horde::applicationUrl('videos/index.php'));
                exit;
            }
        }
    }
}

require VIDEO_TEMPLATES . '/common-header.inc';
require VIDEO_TEMPLATES . '/menu.inc';

$form->renderActive(null, null, null, 'post');

require $registry->get('templates', 'horde') . '/common-footer.inc';
